@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,8 +24,6 @@
 #include <stdio.h>
 #endif
 
-//XXX should remove this mysql specific code from here --alfredo
-#include <mysql.h> // needed for enum_field_types
 #include <boost/scoped_array.hpp>
 
 #include "db_object_helpers.h"
@@ -79,7 +77,7 @@ db_SimpleDatatypeRef CatalogHelper::get_datatype(grt::ListRef<db_SimpleDatatype>
 {
   for (size_t c= types.count(), i= 0; i < c; i++)
   {
-    if (g_strcasecmp(types[i]->name().c_str(), name.c_str())==0)
+    if (base::string_compare(types[i]->name(), name, false)==0)
       return types[i];
     //for (size_t d= types[i]->synonyms().count(), j= 0; j < d; j++)
     //{
@@ -318,7 +316,8 @@ db_TableRef TableHelper::create_associative_table(const db_SchemaRef &schema,
   first_fk->index(index);
   atable->indices().insert(index);
 
-  atable->lastChangeDate(grt::StringRef(fmttime(0, DATETIME_FMT)));
+  atable->createDate(grt::StringRef(fmttime(0, DATETIME_FMT)));
+  atable->lastChangeDate(atable->createDate());
   
   undo.end("Create Associative Table");
 
@@ -952,7 +951,7 @@ static void split_comment(const std::string &comment, size_t db_comment_len,
 {
   size_t res;
   // XXX: check for Unicode line breaks! especially asian languages don't use the ANSI new line.
-  gchar* pointer_to_linebreak = g_utf8_strrchr(comment.c_str(), comment.size(), '\n');
+  gchar* pointer_to_linebreak = g_utf8_strchr(comment.c_str(), comment.size(), '\n');
 
   // We need the number of characters which the string part includes, so convert to a char count.
   if (pointer_to_linebreak != NULL)
@@ -1054,7 +1053,7 @@ ColumnTypeCompareResult ColumnHelper::compare_column_types(const db_ColumnRef &f
 
 void ColumnHelper::set_default_value(db_ColumnRef column, const std::string &value)
 {
-  column->defaultValueIsNull(g_strcasecmp(value.c_str(), "NULL") ? 0 : 1);
+  column->defaultValueIsNull(base::string_compare(value, "NULL", false) ? 0 : 1);
   column->defaultValue(value.c_str());
 
   // If a default value of NULL was set then the column can no longer be a not-null column.
@@ -1255,7 +1254,7 @@ static db_SimpleDatatypeRef findType(const grt::ListRef<db_SimpleDatatype> &type
 {
   for (size_t c= types.count(), i= 0; i < c; i++)
   {
-    if (g_strcasecmp(types[i]->name().c_str(), name.c_str())==0)
+    if (base::string_compare(types[i]->name(), name, false)==0)
     {
       if (!target_version.is_valid() || CatalogHelper::is_type_valid_for_version(types[i], target_version))
         return types[i];
@@ -1467,7 +1466,7 @@ bool bec::parseType(const std::string &type,
         {
             db_UserDatatypeRef utype(user_types[i]);
 
-            if (g_strcasecmp(utype->name().c_str(), typeName.c_str()) == 0)
+            if (base::string_compare(utype->name(), typeName, false) == 0)
             {
                 userType= utype;
                 break;
@@ -1501,7 +1500,6 @@ bool bec::parseType(const std::string &type,
             return false;
 
         simpleType = db_SimpleDatatypeRef();
-        userType = userType;
         if (!overriden_args)
         {
             precision = bec::EMPTY_COLUMN_PRECISION;
@@ -1522,6 +1520,35 @@ bool bec::parseType(const std::string &type,
 }
 
 
+
+std::string bec::get_default_collation_for_charset(const db_SchemaRef &schema, const std::string &character_set)
+{
+  if (schema->owner().is_valid())
+  {
+    db_CatalogRef catalog(db_CatalogRef::cast_from(schema->owner()));
+    db_CharacterSetRef charset = find_named_object_in_list(catalog->characterSets(), character_set);
+    if (charset.is_valid())
+    {
+      return charset->defaultCollation();
+    }
+  }
+  else
+    log_warning("While checking diff, catalog ref was found to be invalid\n");
+
+
+  return "";
+}
+
+std::string bec::get_default_collation_for_charset(const db_TableRef &table, const std::string &character_set)
+{
+  if (table->owner().is_valid())
+    return bec::get_default_collation_for_charset(db_SchemaRef::cast_from(table->owner()), character_set);
+  else
+    log_warning("While checking diff, table ref was found to be invalid\n");
+  return "";
+}
+
+
 std::string bec::TableHelper::generate_comment_text(const std::string& comment_text, size_t comment_lenght)
 {
   std::string comment, leftover;
@@ -1538,4 +1565,4 @@ std::string bec::TableHelper::generate_comment_text(const std::string& comment_t
       .append("*/");
   }
   return comment; 
-}
+}
\ No newline at end of file
