@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,6 +35,9 @@
 
 #include "base/log.h"
 
+#include "mforms/box.h"
+#include "mforms/form.h"
+#include "mforms/toolbar.h"
 #include "mforms/utilities.h"
 
 using namespace bec;
@@ -649,6 +652,23 @@ bool TableColumnsListBE::set_field(const NodeId &node, int column, const std::st
       AutoUndoEdit undo(_owner, col, "defaultValue");
 
       std::string value_= base::trim_right(value);
+
+      db_SimpleDatatypeRef sData;
+
+      if (col->userType().is_valid()) //Get SimpleData from userType when in modeling.
+        sData = col->userType()->actualType();
+      else if(col->simpleType().is_valid()) //When in SQLIDE we use simpleType directly.
+        sData = col->simpleType();
+
+      //Change true to 1 and false to 0 only when user selected numeric type and it's natural number (INT based like TINY/MEDIUM/BIG).
+      if (!value_.empty() && sData.is_valid() && sData->group().is_valid() && sData->group()->name() == "numeric" && sData->numericScale() == 0)
+      {
+        if (same_string(value_, "true", false))
+          value_ = "1";
+        else if (same_string(value_, "false", false))
+          value_ = "0";
+      }
+
       bec::ColumnHelper::set_default_value(col, value_.empty() ? "" : quote_value_if_needed(col, value_));
       _owner->update_change_date();
 
@@ -717,6 +737,10 @@ bool TableColumnsListBE::set_field(const NodeId &node, int column, const std::st
   return false;
 }
 
+void TableColumnsListBE::reset_placeholder()
+{
+  _editing_placeholder_row = -1;
+}
 
 bool TableColumnsListBE::set_field(const NodeId &node, int column, int value)
 {
@@ -1704,24 +1728,37 @@ NodeId IndexListBE::add_column(const db_ColumnRef &column, const db_IndexRef &aI
   if (!index.is_valid())
     return NodeId();
 
-  if (!index_editable(index) || index_belongs_to_fk(index).is_valid())
+  if (/*!index_editable(index) || */index_belongs_to_fk(index).is_valid())
     return NodeId();
 
-  std::string column_struct= index.get_metaclass()->get_member_type("columns").content.object_class;
-  db_IndexColumnRef icolumn=
-    _owner->get_grt()->create_object<db_IndexColumn>(column_struct);
-  icolumn->owner(index);
-  icolumn->referencedColumn(column);
+  // special handling for PK columns
+  if (strcmp(index->indexType().c_str(), "PRIMARY") == 0)
+  {
+    AutoUndoEdit undo(_owner);
+    _owner->get_table()->addPrimaryKeyColumn(column);
+    _owner->update_change_date();
 
-  AutoUndoEdit undo(_owner);
+    undo.end(
+       strfmt(_("Add column '%s' to primary key from '%s'"),
+              column->name().c_str(), _owner->get_name().c_str()));
+  }
+  else
+  {
+    std::string column_struct= index.get_metaclass()->get_member_type("columns").content.object_class;
+    db_IndexColumnRef icolumn=
+      _owner->get_grt()->create_object<db_IndexColumn>(column_struct);
+    icolumn->owner(index);
+    icolumn->referencedColumn(column);
 
-  index->columns().insert(icolumn);
-  _owner->update_change_date();
+    AutoUndoEdit undo(_owner);
 
-  undo.end(
-      strfmt(_("Add column '%s' to index '%s.%s'"), 
-        column->name().c_str(), _owner->get_name().c_str(), index->name().c_str()));
+    index->columns().insert(icolumn);
+    _owner->update_change_date();
 
+    undo.end(
+        strfmt(_("Add column '%s' to index '%s.%s'"), 
+          column->name().c_str(), _owner->get_name().c_str(), index->name().c_str()));
+  }
   _column_list.refresh();
   return NodeId(index->columns().count()-1);
 }
@@ -1732,25 +1769,39 @@ void IndexListBE::remove_column(const NodeId &node)
   RefreshUI::Blocker __centry(*_owner);
 
   db_IndexRef index= get_selected_index();
-  if (!index_editable(index) || index_belongs_to_fk(index).is_valid())
+  if (/*!index_editable(index) || */index_belongs_to_fk(index).is_valid())
     return;
 
   db_ColumnRef column(_owner->get_table()->columns().get(node[0]));
 
-  for (size_t c= index->columns().count(), i= 0; i < c; i++)
+  // special handling for PK columns
+  if (strcmp(index->indexType().c_str(), "PRIMARY") == 0)
   {
-    if (index->columns().get(i)->referencedColumn() == column)
+    AutoUndoEdit undo(_owner);
+    _owner->get_table()->removePrimaryKeyColumn(column);
+    _owner->update_change_date();
+
+    undo.end(
+       strfmt(_("Remove column '%s' from primary key from '%s'"),
+              column->name().c_str(), _owner->get_name().c_str()));
+  }
+  else
+  {
+    for (size_t c= index->columns().count(), i= 0; i < c; i++)
     {
-      AutoUndoEdit undo(_owner);
+      if (index->columns().get(i)->referencedColumn() == column)
+      {
+        AutoUndoEdit undo(_owner);
 
-      index->columns().remove(i);
-      _owner->update_change_date();
+        index->columns().remove(i);
+        _owner->update_change_date();
 
-      undo.end(
-        strfmt(_("Remove column '%s' from index '%s.%s'"), 
-            column->name().c_str(), _owner->get_name().c_str(), index->name().c_str()));
-      _column_list.refresh();
-      break;
+        undo.end(
+                 strfmt(_("Remove column '%s' from index '%s.%s'"),
+                        column->name().c_str(), _owner->get_name().c_str(), index->name().c_str()));
+        _column_list.refresh();
+        break;
+      }
     }
   }
 }
@@ -2501,6 +2552,42 @@ bool FKConstraintListBE::set_field(const NodeId &node, int column, int value)
   return false;
 }
 
+static bool check_if_null(TableEditorBE *_owner, db_ForeignKeyRef &fk, std::string caption)
+{
+  bool one_col_is_not_null = false;
+  for (size_t i = 0; i < fk->columns().count(); i++)
+  {
+    if (fk->columns()[i]->isNotNull())
+    {
+      one_col_is_not_null = true;
+      break;
+    }
+  }
+
+  if (one_col_is_not_null)
+  {
+    int ret_val = mforms::Utilities::show_warning(caption,
+      _("You can't use a SET NULL action if one of the referencing columns is set to NOT NULL.\n"
+        "Would you like to revert the change or remove the NOT NULL from the column attribute?"),
+      _("Remove NOT NULL"), _("Revert"));
+
+    if (ret_val == mforms::ResultCancel)
+      return false;
+
+    AutoUndoEdit undo(_owner);
+    for (size_t i =0; i < fk->columns().count(); i++)
+    {
+      fk->columns()[i]->isNotNull(false);
+    }
+    _owner->update_change_date();
+    (*_owner->get_table()->signal_refreshDisplay())("column");
+    undo.end(
+        strfmt(_("Remove NOT NULL for columns '%s'"), fk->owner()->name().c_str()));
+      _owner->do_ui_refresh();
+  }
+  return true;
+}
+
 bool FKConstraintListBE::set_field(const NodeId &node, int column, const std::string &value)
 {
   db_ForeignKeyRef fk;
@@ -2531,6 +2618,11 @@ bool FKConstraintListBE::set_field(const NodeId &node, int column, const std::st
   case OnDelete:
     if (fk->deleteRule() != value)
     {
+      if (value.find("SET NULL") != std::string::npos)
+      {
+        if (!check_if_null(_owner, fk, _("On Delete Action")))
+          return false;
+      }
       AutoUndoEdit undo(_owner);
 
       fk->deleteRule(value);
@@ -2543,6 +2635,11 @@ bool FKConstraintListBE::set_field(const NodeId &node, int column, const std::st
   case OnUpdate:
     if (fk->updateRule() != value)
     {
+      if (value.find("SET NULL") != std::string::npos)
+      {
+        if (!check_if_null(_owner, fk, _("On Update Action")))
+          return false;
+      }
       AutoUndoEdit undo(_owner);
 
       fk->updateRule(value);
@@ -2796,6 +2893,8 @@ DBObjectEditorBE(grtm, table, rdbms),
 _table(table),
 _fk_list(this)
 {
+  _inserts_panel = NULL;
+
   if (table.class_name() == "db.Table") throw std::logic_error("table object is abstract");
   
   scoped_connect(table->signal_refreshDisplay(),(boost::bind(&BaseEditor::on_object_changed, this)));
@@ -2851,6 +2950,7 @@ NodeId TableEditorBE::add_column(const std::string &name)
 NodeId TableEditorBE::duplicate_column(const db_ColumnRef &column, int insert_after)
 {
   db_ColumnRef new_column = shallow_copy_object(column);
+  new_column->oldName(""); //It's new column, so it shouldn't have any old name set.
 
   std::string name = new_column->name();
   std::string new_name = name;
@@ -3159,47 +3259,63 @@ Recordset::Ref TableEditorBE::get_inserts_model()
     _inserts_storage->table(_table);
 
     _inserts_model= Recordset::create(_grtm);
-    _inserts_model->show_apply_buttons(true);
+    _inserts_model->set_inserts_editor(true);
     _inserts_model->data_storage(_inserts_storage);
     _inserts_model->refresh();
-    
-    _inserts_model->action_list().register_action("record_export",
-                                                  boost::bind(&TableEditorBE::show_export_wizard, this, (mforms::Form*)0));
-    _inserts_model->action_list().register_action("record_import",
-                                                  boost::bind(&TableEditorBE::show_import_wizard, this));
-
   }
   return _inserts_model;
 }
 
+
+mforms::View *TableEditorBE::create_inserts_panel(mforms::View *grid)
+{
+  Recordset::Ref rset = get_inserts_model();
+
+  mforms::ToolBar *tbar = rset->get_toolbar();
+  tbar->find_item("record_export")->signal_activated()->connect(boost::bind(&TableEditorBE::show_export_wizard, this, (mforms::Form*)0));
+  if (tbar->find_item("record_import"))
+    tbar->find_item("record_import")->signal_activated()->connect(boost::bind(&TableEditorBE::show_import_wizard, this));
+
+  _inserts_panel = mforms::manage(new mforms::Box(false));
+  _inserts_panel->add(mforms::manage(tbar), false, true);
+  _inserts_panel->add(mforms::manage(grid), true, true);
+
+  return _inserts_panel;
+}
+
 //--------------------------------------------------------------------------------------------------
 
 void TableEditorBE::show_export_wizard(mforms::Form *owner)
 {
-  grt::ValueRef option(_grtm->get_app_option("TableEditor:LastExportDirectory"));
-  std::string path = option.is_valid() ? grt::StringRef::cast_from(option) : "";
-  option = _grtm->get_app_option("TableEditor:LastExportExtension");
-  std::string extension = option.is_valid() ? grt::StringRef::cast_from(option) : "";
-  InsertsExportForm exporter(owner, _inserts_model, extension);
-  exporter.set_title(strfmt(_("Export Inserts for %s"), get_name().c_str()));
-  if (!path.empty())
+  if (_inserts_model && _inserts_model->count() > 0)
   {
-    path = bec::make_path(path, get_name());
-    exporter.set_path(path);
-  }  
-  path = exporter.run();
-  if (path.empty())
-    _grtm->replace_status_text(_("Export inserts canceled"));
-  else
-  {
-    _grtm->replace_status_text(strfmt(_("Exported inserts to %s"), path.c_str()));
-    _grtm->set_app_option("TableEditor:LastExportDirectory", grt::StringRef(exporter.get_directory()));
-    extension = base::extension(path);
-    if (!extension.empty() && extension[0] == '.')
-      extension = extension.substr(1);
-    if (!extension.empty())
-      _grtm->set_app_option("TableEditor:LastExportExtension", grt::StringRef(extension));
+    grt::ValueRef option(_grtm->get_app_option("TableEditor:LastExportDirectory"));
+    std::string path = option.is_valid() ? grt::StringRef::cast_from(option) : "";
+    option = _grtm->get_app_option("TableEditor:LastExportExtension");
+    std::string extension = option.is_valid() ? grt::StringRef::cast_from(option) : "";
+    InsertsExportForm exporter(owner, _inserts_model, extension);
+    exporter.set_title(strfmt(_("Export Inserts for %s"), get_name().c_str()));
+    if (!path.empty())
+    {
+      path = bec::make_path(path, get_name());
+      exporter.set_path(path);
+    }
+    path = exporter.run();
+    if (path.empty())
+      _grtm->replace_status_text(_("Export inserts canceled"));
+    else
+    {
+      _grtm->replace_status_text(strfmt(_("Exported inserts to %s"), path.c_str()));
+      _grtm->set_app_option("TableEditor:LastExportDirectory", grt::StringRef(exporter.get_directory()));
+      extension = base::extension(path);
+      if (!extension.empty() && extension[0] == '.')
+        extension = extension.substr(1);
+      if (!extension.empty())
+        _grtm->set_app_option("TableEditor:LastExportExtension", grt::StringRef(extension));
+    }
   }
+  else
+    mforms::Utilities::show_message("Export Data", "No data to be exported.", "OK");
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -3209,7 +3325,7 @@ void TableEditorBE::show_import_wizard()
   grt::BaseListRef args(_grtm->get_grt());
   
   db_TableRef table(get_table());
-  if (table.is_valid())
+  if (table.is_valid() && table->columns().count() > 0)
   {
     args.ginsert(grtwrap_editablerecordset(table, _inserts_model));
 
