@@ -573,3 +573,77 @@ void gtk_reparent_realized(Gtk::Widget *widget, Gtk::Container *new_parent)
   new_parent->add(*widget);
   widget->unreference();
 }
+
+//------------------------------------------------------------------------------
+void PanedConstrainer::size_alloc(Gtk::Allocation &_alloc)
+{
+  if (_reentrant)
+    return;
+  _reentrant = true;
+
+  if (_pan && (_margin_min > 0 || _margin_max > 0))
+  {
+    if (_pan->get_position() <= _margin_min)
+      _pan->set_position(0);
+    else if ( _margin_max >= ((_vertical ? _pan->get_height() : _pan->get_width()) - _pan->get_position()))
+        _pan->set_position(_pan->property_max_position());
+  }
+
+  _reentrant = false;
+}
+
+PanedConstrainer::PanedConstrainer(Gtk::Paned *pan) : _pan(pan), _vertical(true)
+{
+  _reentrant = false;
+  _margin_min = 60;
+  _margin_max = 60;
+
+  if (_pan)
+  {
+
+    Gtk::VPaned* vpan = dynamic_cast<Gtk::VPaned*>(pan);
+    if (vpan)
+      _vertical = true;
+    else
+      _vertical = false;
+
+    _size_alloc_sig = _pan->signal_size_allocate().connect(sigc::mem_fun(this, &PanedConstrainer::size_alloc));
+  }
+}
+void PanedConstrainer::make_constrainer(Gtk::Paned *paned, int min_size, int max_size)
+{
+  if (paned)
+  {
+    PanedConstrainer *pc = new PanedConstrainer(paned);
+    pc->set_margin(min_size, max_size);
+    paned->set_data("paned_constrainer", pc);
+    paned->add_destroy_notify_callback(reinterpret_cast<void*>(pc), &PanedConstrainer::destroy);
+  }
+}
+
+PanedConstrainer::~PanedConstrainer()
+{
+  _size_alloc_sig.disconnect();
+}
+
+void *PanedConstrainer::destroy(void *data)
+{
+  PanedConstrainer *pc = reinterpret_cast<PanedConstrainer*>(data);
+  if (pc)
+    delete pc;
+
+
+  return 0;
+}
+
+void PanedConstrainer::set_margin(int min, int max)
+{
+  _margin_min = min;
+  _margin_max = max;
+}
+
+Gtk::Paned* PanedConstrainer::get()
+{
+  return _pan;
+}
+
