@@ -187,8 +187,6 @@ GRT::GRT()
   _default_undo_manager= new UndoManager(this);
 
   add_module_loader(new CPPModuleLoader(this));
-
-  g_static_rec_mutex_init(&_message_mutex);
   
   // register metaclass for base class
   add_metaclass(MetaClass::create_base_class(this));
@@ -226,8 +224,6 @@ GRT::~GRT()
        iter != _metaclasses.end(); ++iter)
     delete iter->second;
   _metaclasses.clear();
-
-  g_static_rec_mutex_free(&_message_mutex);
 }
 
 void GRT::push_undo_manager(UndoManager *um)
@@ -686,10 +682,10 @@ bool GRT::load_module(const std::string &path, bool refresh)
           else
             register_new_module(module);
         }
-        catch (...)
+        catch (std::exception &exc)
         {
-         // crashes in some systems, works in others
-         // delete module;
+          log_debug("Deleting module %s because of %s\n", module->name().c_str(), exc.what());
+          delete module;
           throw;
         }
         return true;
@@ -928,6 +924,9 @@ void GRT::refresh_module(Module *module)
 
 void GRT::register_new_interface(Interface *iface)
 {
+  if (get_interface(iface->name()))
+      throw std::logic_error("Duplicate interface "+iface->name());
+
   _interfaces[iface->name()]= iface;
 } 
 
@@ -1006,14 +1005,14 @@ Shell *GRT::get_shell()
 
 void GRT::push_message_handler(const MessageSlot &slot)
 {
-  GStaticRecMutexLock lock(_message_mutex);  
+  base::RecMutexLock lock(_message_mutex);
   _message_slot_stack.push_back(slot);
 }
 
 
 void GRT::pop_message_handler()
 {
-  GStaticRecMutexLock lock(_message_mutex);
+  base::RecMutexLock lock(_message_mutex);
   if (_message_slot_stack.empty())
     log_error("pop_message_handler() called on empty handler stack");
   else
@@ -1032,7 +1031,7 @@ bool GRT::handle_message(const Message &msg, void *sender)
     for (;;)
     {
       {
-        GStaticRecMutexLock lock(_message_mutex);
+        base::RecMutexLock lock(_message_mutex);
         if ((int)_message_slot_stack.size() - i - 1 >= 0)
         {
           slot = _message_slot_stack[_message_slot_stack.size() - i - 1];
@@ -1073,7 +1072,7 @@ bool GRT::query_status()
 
 void GRT::make_output_visible(void *sender)
 {
-  g_static_rec_mutex_lock(&_message_mutex);
+  base::RecMutexLock lock(_message_mutex);
   Message msg;
   msg.type= ControlMsg;
   msg.text= "show";
@@ -1081,13 +1080,12 @@ void GRT::make_output_visible(void *sender)
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
   handle_message(msg, sender);
-  g_static_rec_mutex_unlock(&_message_mutex);
 }
 
 // XXX: these handlers should go and be replaced by pure log_* calls.
 void GRT::send_error(const std::string &message, const std::string &details, void *sender)
 {
-  g_static_rec_mutex_lock(&_message_mutex);
+  base::RecMutexLock lock(_message_mutex);
   Message msg;
   msg.type= ErrorMsg;
   msg.text= message;
@@ -1095,7 +1093,6 @@ void GRT::send_error(const std::string &message, const std::string &details, voi
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
   handle_message(msg, sender);
-  g_static_rec_mutex_unlock(&_message_mutex);
   
   log_error("%s\t%s\n", message.c_str(), details.c_str());
 }
@@ -1103,7 +1100,7 @@ void GRT::send_error(const std::string &message, const std::string &details, voi
 
 void GRT::send_warning(const std::string &message, const std::string &details, void *sender)
 {
-  g_static_rec_mutex_lock(&_message_mutex);
+  base::RecMutexLock lock(_message_mutex);
   Message msg;
   msg.type= WarningMsg;
   msg.text= message;
@@ -1111,7 +1108,6 @@ void GRT::send_warning(const std::string &message, const std::string &details, v
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
   handle_message(msg, sender);
-  g_static_rec_mutex_unlock(&_message_mutex);
   
   log_warning("%s\t%s", message.c_str(), details.c_str());
 }
@@ -1119,7 +1115,7 @@ void GRT::send_warning(const std::string &message, const std::string &details, v
 
 void GRT::send_info(const std::string &message, const std::string &details, void *sender)
 {
-  g_static_rec_mutex_lock(&_message_mutex);
+  base::RecMutexLock lock(_message_mutex);
   Message msg;
   msg.type= InfoMsg;
   msg.text= message;
@@ -1127,7 +1123,6 @@ void GRT::send_info(const std::string &message, const std::string &details, void
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
   handle_message(msg, sender);
-  g_static_rec_mutex_unlock(&_message_mutex);
   
   log_info("%s\t%s\n", message.c_str(), details.c_str());
 }
@@ -1165,7 +1160,7 @@ void GRT::end_progress_step()
 
 void GRT::send_progress(float percentage, const std::string &message, const std::string &details, void *sender)
 {
-  g_static_rec_mutex_lock(&_message_mutex);
+  base::RecMutexLock lock(_message_mutex);
   Message msg;
   msg.type= ProgressMsg;
   msg.text= message;
@@ -1182,7 +1177,6 @@ void GRT::send_progress(float percentage, const std::string &message, const std:
   }
   msg.progress= percentage;
   handle_message(msg, sender);
-  g_static_rec_mutex_unlock(&_message_mutex);
 
   // Progress messages are often sent without a real message, just to trigger some actions.
   // Don't log empty lines.
@@ -1193,7 +1187,7 @@ void GRT::send_progress(float percentage, const std::string &message, const std:
 
 void GRT::send_verbose(const std::string &message, void *sender)
 {
-  g_static_rec_mutex_lock(&_message_mutex);
+  base::RecMutexLock lock(_message_mutex);
   Message msg;
   msg.type= VerboseMsg;
   msg.text= message;
@@ -1201,7 +1195,6 @@ void GRT::send_verbose(const std::string &message, void *sender)
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
   handle_message(msg, sender);
-  g_static_rec_mutex_unlock(&_message_mutex);
 
   log_debug2("%s", message.c_str());
 }
@@ -1209,7 +1202,7 @@ void GRT::send_verbose(const std::string &message, void *sender)
 
 void GRT::send_output(const std::string &message, void *sender)
 {
-  g_static_rec_mutex_lock(&_message_mutex);
+  base::RecMutexLock lock(_message_mutex);
   Message msg;
   msg.type= OutputMsg;
   msg.text= message;
@@ -1217,7 +1210,6 @@ void GRT::send_output(const std::string &message, void *sender)
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
   handle_message(msg, sender);
-  g_static_rec_mutex_unlock(&_message_mutex);
 
   // Log send_output only when verbose is on to avoid duplicate prints to stdout, when logged text also goes to stderr/out
   // TODO: fix the actual cause for sending duplicates and remove _verbose (doesn't fit to our logging).
