@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -732,9 +732,14 @@ void Mysql_sql_parser::process_field_type_item(const SqlAstNode *item, db_mysql_
       const SqlAstNode* float_item = item->subitem(sql::_float_options);
       if (float_item)
         process_float_options_item(float_item, &precision, &scale);
-      const SqlAstNode* real_item = item->subitem(sql::_real_type);
-      if (real_item)
-        process_float_options_item(real_item, &precision, &scale);
+      const SqlAstNode* precision_item = item->subitem(sql::_opt_precision);
+      if (precision_item == NULL)
+      {
+        static sql::symbol precision_path[] = { sql::_real_type, sql::_precision, sql::_ };
+        precision_item = item->subitem_by_path(precision_path);
+      }
+      if (precision_item)
+        process_float_options_item(precision_item, &precision, &scale);
       if (!scale.empty())
         SET_INT(column->scale, scale)
       if (!precision.empty())
@@ -844,9 +849,9 @@ void Mysql_sql_parser::process_field_attributes_item(const SqlAstNode *item, db_
             column->defaultValueIsNull(1);
           explicitDefaultValue= true;
         }
-        // this also handles the 'ON UPDATE CURRENT_TIMESTAMP' case as
-        // CURRENT_TIMESTAMP is assigned by lexer the same token number as NOW_SYM
-        else if ((aux_item= subitem->subseq(sql::_ON, sql::_UPDATE_SYM, sql::_NOW_SYM)))
+        // This also handles the 'ON UPDATE CURRENT_TIMESTAMP' case as
+        // CURRENT_TIMESTAMP is a synonymof NOW.
+        else if ((aux_item = subitem->subseq(sql::_ON, sql::_UPDATE_SYM, sql::_now)))
         {
           std::string text;
           // it was decided that 'ON UPDATE CURRENT_TIMESTAMP' should be stored in defaultValue to not create new rarely-used field
@@ -856,9 +861,11 @@ void Mysql_sql_parser::process_field_attributes_item(const SqlAstNode *item, db_
             text.append(" ");
           }
 
+          // NOW is a subtree now with optional precision.
+          aux_item = subitem->subitem(sql::_now);
           text.append(subitem->subitem(sql::_ON)->value()).append(" ")
             .append(subitem->subitem(sql::_UPDATE_SYM)->value()).append(" ")
-            .append(subitem->subitem(sql::_NOW_SYM)->value());
+            .append(aux_item->subitem(sql::_NOW_SYM)->value());
 
           bec::ColumnHelper::set_default_value(column, text);
         }
@@ -2679,12 +2686,7 @@ grt::Ref<T> Mysql_sql_parser::create_or_find_named_routine(const grt::ListRef<T>
       grt::Ref<T> item(obj_list[i]);
       if (*item->routineType() == routine_type)
       {
-        if (case_sensitive && item->name() == obj_name)
-        {
-          obj = item;
-          break;
-        }
-        else if (!case_sensitive && g_strcasecmp(item->name().c_str(), obj_name.c_str()) == 0)
+        if(base::string_compare(item->name(), obj_name, case_sensitive) == 0)
         {
           obj = item;
           break;
@@ -2707,4 +2709,4 @@ grt::Ref<T> Mysql_sql_parser::create_or_find_named_routine(const grt::ListRef<T>
   try { obj.set_member("lastChangeDate", StringRef(time)); } catch (std::exception&) {}
   
   return obj;
-}
\ No newline at end of file
+}
